from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import UserCreationForm, UserChangeForm
from .models import User, TelegramAuthCode


class AuthTgUserCreationForm(UserCreationForm):
    class Meta:
        model = User
        fields = ('username', 'tg_id', 'tg_username', 'tg_firstname', 'tg_lastname')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields.pop('password1', None)
        self.fields.pop('password2', None)


class AuthTgUserChangeForm(UserChangeForm):
    class Meta:
        model = User
        fields = ('username', 'tg_id', 'tg_username', 'tg_firstname', 'tg_lastname',
                  'tg_photo', 'is_active', 'is_staff', 'is_superuser')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields.pop('password', None)


class AuthTgUserAdmin(UserAdmin):
    form = AuthTgUserChangeForm
    add_form = AuthTgUserCreationForm

    list_display = ('username', 'tg_id', 'tg_username', 'tg_firstname',
                    'tg_lastname', 'is_staff', 'is_active', 'created_at')
    list_filter = ('is_staff', 'is_superuser', 'is_active', 'created_at')
    search_fields = ('username', 'tg_username', 'tg_firstname', 'tg_lastname', 'tg_id')
    ordering = ('-created_at',)

    fieldsets = (
        ('Основная информация', {
            'fields': ('username', 'tg_id')
        }),
        ('Telegram информация', {
            'fields': ('tg_username', 'tg_firstname', 'tg_lastname', 'tg_photo')
        }),
        ('Права доступа', {
            'fields': ('is_active', 'is_staff', 'is_superuser', 'groups', 'user_permissions'),
        }),
        ('Важные даты', {
            'fields': ('created_at', 'last_login'),
        }),
    )

    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('username', 'tg_id', 'tg_username', 'tg_firstname', 'tg_lastname',
                       'is_staff', 'is_superuser', 'is_active'),
        }),
    )

    readonly_fields = ('created_at', 'last_login',)

    def get_readonly_fields(self, request, obj=None):
        if obj:
            return self.readonly_fields + ('tg_id',)
        return self.readonly_fields


class TelegramAuthCodeAdmin(admin.ModelAdmin):
    list_display = ('auth_code', 'created_at', 'is_used', 'user')
    list_filter = ('is_used', 'created_at')
    search_fields = ('auth_code', 'user__username', 'user__tg_username')
    readonly_fields = ('auth_code', 'created_at')

    fieldsets = (
        (None, {
            'fields': ('auth_code', 'user', 'is_used', 'created_at')
        }),
    )


admin.site.register(User, AuthTgUserAdmin)
admin.site.register(TelegramAuthCode, TelegramAuthCodeAdmin)


from django.contrib.auth.backends import ModelBackend
from django.contrib.auth import get_user_model

class TelegramAuthBackend(ModelBackend):
    def authenticate(self, request, username=None, **kwargs):
        UserModel = get_user_model()
        try:
            user = UserModel.objects.get(username=username)
            return user
        except UserModel.DoesNotExist:
            try:
                user = UserModel.objects.get(tg_id=username)
                return user
            except (UserModel.DoesNotExist, ValueError):
                return None

    def get_user(self, user_id):
        UserModel = get_user_model()
        try:
            return UserModel.objects.get(pk=user_id)
        except UserModel.DoesNotExist:
            return None


from django.contrib.auth.base_user import BaseUserManager



class AuthTgUserManager(BaseUserManager):
    def create_user(self, username, password=None, **extra_fields):
        if not username:
            raise ValueError('Username must be set')
        user = self.model(username=username, **extra_fields)
        if password:
            user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, username, password=None, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)
        extra_fields['tg_id'] = 1

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self.create_user(username, password, **extra_fields)


from django.contrib.auth import authenticate, login, logout
from django.conf import settings
from django.views.generic import TemplateView

from .forms import TelegramAuthenticationForm
from django.shortcuts import redirect, render
from django.views import View
from django.views.decorators.csrf import csrf_protect
from rest_framework.views import APIView
from rest_framework.response import Response
from django.http import JsonResponse, HttpResponseRedirect, HttpResponse
from .models import TelegramAuthCode, User
import asyncio
import requests

SESSION_COOKIE_AGE = getattr(settings, 'SESSION_COOKIE_AGE', 60 * 60 * 24 * 7)
try:
    BOT_TOKEN = settings.BOT_TOKEN
    BOT_USERNAME = settings.BOT_USERNAME
except Exception:
    raise 'Необходимо указать в settings.py BOT_TOKEN и BOT_USERNAME'


class TgAuthProfilePhotoView(View):
    def get(self, request, file_path):
        try:
            if not request.session.get('user_id'):
                return HttpResponse(status=401)

            telegram_url = f"https://api.telegram.org/file/bot{BOT_TOKEN}/{file_path}"
            response = requests.get(telegram_url)

            if response.status_code == 200:
                return HttpResponse(
                    response.content,
                    content_type=response.headers.get('Content-Type', 'image/jpeg')
                )
            else:
                return HttpResponse(status=404)
        except Exception as e:
            print(f"Error fetching profile photo: {e}")
            return HttpResponse(status=500)


class TgAuthLogoutView(View):
    def get(self, request):
        request.session.flush()
        logout(request)
        return redirect('home')


class TgAuthTelegramLoginView(APIView):
    def get(self, request):
        auth_record = TelegramAuthCode.objects.create()
        bot_link = f"https://t.me/{BOT_USERNAME}?start={auth_record.auth_code}"
        return Response({
            'auth_url': bot_link,
            'auth_code': str(auth_record.auth_code)
        })


@csrf_protect
def check_auth_status(request):
    auth_code = request.GET.get('auth_code')
    if not auth_code:
        return JsonResponse({'authenticated': False})

    try:
        auth_record = TelegramAuthCode.objects.select_related('user').get(
            auth_code=auth_code,
            is_used=True,
            user__isnull=False
        )

        user = auth_record.user
        authenticated_user = authenticate(request, username=user.username)
        if authenticated_user:
            login(request, authenticated_user)

            request.session['user_id'] = auth_record.user.id
            request.session.set_expiry(SESSION_COOKIE_AGE)

            asyncio.run(delete_old_codes(user.id))

            return JsonResponse({
                'authenticated': True
            })

        return JsonResponse({'authenticated': False})

    except TelegramAuthCode.DoesNotExist:
        return JsonResponse({'authenticated': False})


async def delete_old_codes(user_id):
    try:
        await TelegramAuthCode.objects.filter(user_id=user_id).adelete()
        return True
    except Exception:
        return False


class TgAuthTelegramCompleteView(View):
    def get(self, request):
        if not request.session.get('user_id'):
            return redirect('home')
        return HttpResponseRedirect('/profile/')


def telegram_admin_login(request):
    if request.method == 'POST':
        form = TelegramAuthenticationForm(request.POST)
        if form.is_valid():
            username = form.cleaned_data['username']
            user = authenticate(request, username=username)
            if user and user.is_staff:
                login(request, user)
                request.session['user_id'] = user.id
                request.session['is_admin'] = True
                return redirect('admin:index')
    else:
        form = TelegramAuthenticationForm()

    return render(request, 'admin/login.html', {'form': form})


# class TgAuthHomeView(TemplateView):
#     template_name = 'auth_tg/home.html'
#
#     def get_context_data(self, **kwargs):
#         context = super().get_context_data(**kwargs)
#         context['is_authenticated'] = self.request.session.get('user_id') is not None
#         return context


# class TgAuthProfileView(TemplateView):
#     template_name = 'auth_tg/profile.html'
#
#     def get(self, request, *args, **kwargs):
#         if not request.session.get('user_id'):
#             return redirect('home')
#
#         try:
#             user = User.objects.get(id=request.session['user_id'])
#             photo_path = user.get_photo_path()
#
#             return render(request, self.template_name, {
#                 'user': user,
#                 'photo_path': photo_path,
#                 'is_authenticated': True
#             })
#         except User.DoesNotExist:
#             del request.session['user_id']
#             return redirect('home')


class BaseTgAuthHomeView(TemplateView):
    template_name = 'auth_tg/home.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['is_authenticated'] = self.request.session.get('user_id') is not None
        return context


class BaseTgAuthProfileView(TemplateView):
    template_name = 'auth_tg/profile.html'

    def get(self, request, *args, **kwargs):
        if not request.session.get('user_id'):
            return redirect('home')

        try:
            user = User.objects.get(id=request.session['user_id'])
            photo_path = user.get_photo_path()

            return render(request, self.template_name, {
                'user': user,
                'photo_path': photo_path,
                'is_authenticated': True
            })
        except User.DoesNotExist:
            del request.session['user_id']
            return redirect('home')


class TgAuthHomeView(BaseTgAuthHomeView):
    pass


class TgAuthProfileView(BaseTgAuthProfileView):
    pass


import time
import uuid
from datetime import timedelta
from django.utils import timezone
import telebot
from telebot import types
from django.conf import settings
from auth_tg.models import TelegramAuthCode, User


channels = getattr(settings, 'CHANNELS', None)
channels = [int(channel) for channel in channels.split(',')] if channels else []

channel_links = getattr(settings, 'CHANNEL_NAMES', None)
channel_links = [channel for channel in channel_links.split(',')] if channel_links else []
try:
    BOT_TOKEN = settings.BOT_TOKEN
    BOT_USERNAME = settings.BOT_USERNAME
except Exception:
    raise 'Необходимо указать в settings.py BOT_TOKEN и BOT_USERNAME'
class DjangoTelegramBot:
    def __init__(self):
        self.bot = telebot.TeleBot(BOT_TOKEN)
        self.setup_handlers()

    def auth_bot(self):
        try:
            print("Бот запущен и ожидает сообщений...")
            self.bot.polling(none_stop=True, interval=0)
        except Exception as e:
            print(f"Ошибка в работе бота: {e}")
            time.sleep(15)
            self.auth_bot()

    def check_subscribe(self, user_id):
        if not channels or not channel_links:
            return True

        result = []
        for channel in channels:
            try:
                member_status = self.bot.get_chat_member(channel, user_id).status
                result.append(member_status in ['member', 'administrator', 'creator'])
            except telebot.apihelper.ApiException as e:
                print(f"Ошибка при проверке подписки: {e}")
                continue
        return all(result) if result else True

    def create_subscription_keyboard(self):
        keyboard = types.InlineKeyboardMarkup()
        if channel_links:
            for link in channel_links:
                button = types.InlineKeyboardButton(
                    text=f"Подписаться на {link}",
                    url=f"https://t.me/{link[1:]}"
                )
                keyboard.add(button)
            check_button = types.InlineKeyboardButton(
                text="✅ Я подписался",
                callback_data="check_subscription"
            )
            keyboard.add(check_button)
        return keyboard

    def setup_handlers(self):
        @self.bot.callback_query_handler(func=lambda call: call.data == "check_subscription")
        def callback_check_subscription(call):
            if self.check_subscribe(call.from_user.id):
                try:
                    auth_record = TelegramAuthCode.objects.filter(
                        user__tg_id=call.from_user.id,
                        is_used=False,
                        created_at__gte=timezone.now() - timedelta(minutes=30)
                    ).latest('created_at')

                    auth_record.is_used = True
                    auth_record.save()

                    self.bot.edit_message_text(
                        chat_id=call.message.chat.id,
                        message_id=call.message.message_id,
                        text="✅ Авторизация успешна! Можете вернуться на сайт."
                    )
                except TelegramAuthCode.DoesNotExist:
                    self.bot.answer_callback_query(
                        call.id,
                        "Ошибка авторизации. Попробуйте начать сначала.",
                        show_alert=True
                    )
            else:
                self.bot.answer_callback_query(
                    call.id,
                    "Вы не подписались на все каналы!",
                    show_alert=True
                )

        @self.bot.message_handler(commands=['start'])
        def handle_start(message):
            try:
                command_args = message.text.split()
                if len(command_args) != 2:
                    self.bot.reply_to(
                        message,
                        "Для аутентификации используйте специальную ссылку с сайта."
                    )
                    return

                auth_uuid = command_args[1]
                try:
                    uuid_obj = uuid.UUID(auth_uuid)
                except ValueError:
                    self.bot.reply_to(
                        message,
                        "Неверный формат идентификатора аутентификации."
                    )
                    return

                try:
                    auth_record = TelegramAuthCode.objects.get(
                        auth_code=uuid_obj,
                        is_used=False,
                        created_at__gte=timezone.now() - timedelta(minutes=30)
                    )
                except TelegramAuthCode.DoesNotExist:
                    self.bot.reply_to(
                        message,
                        "Код аутентификации недействителен или устарел."
                    )
                    return

                user = message.from_user

                try:
                    photo_url = None
                    try:
                        user_profile_photos = self.bot.get_user_profile_photos(user.id)
                        if user_profile_photos.total_count > 0:
                            file_info = self.bot.get_file(user_profile_photos.photos[0][-1].file_id)
                            photo_url = f"https://api.telegram.org/file/bot{BOT_TOKEN}/{file_info.file_path}"
                    except Exception as photo_error:
                        print(f"Ошибка при получении фото профиля: {photo_error}")

                    tg_user, created = User.objects.update_or_create(
                        tg_id=user.id,
                        defaults={
                            'tg_username': user.username or '',
                            'tg_firstname': user.first_name or '',
                            'tg_lastname': user.last_name or '',
                            'tg_photo': photo_url or '',
                            'username': user.username or str(user.id),
                            'is_active': True
                        }
                    )

                    auth_record.user = tg_user
                    auth_record.save()

                    if channels and channel_links and not self.check_subscribe(user.id):
                        keyboard = self.create_subscription_keyboard()
                        self.bot.reply_to(
                            message,
                            "Для продолжения необходимо подписаться на каналы:",
                            reply_markup=keyboard
                        )
                        return

                    auth_record.is_used = True
                    auth_record.save()

                    self.bot.reply_to(
                        message,
                        "✅ Авторизация успешна! Можете вернуться на сайт."
                    )

                except Exception as db_error:
                    print(f"Ошибка при работе с базой данных: {db_error}")
                    self.bot.reply_to(
                        message,
                        "❌ Произошла ошибка при сохранении данных. Пожалуйста, попробуйте позже."
                    )

            except Exception as e:
                print(f"Неожиданная ошибка: {e}")
                self.bot.reply_to(
                    message,
                    "❌ Произошла ошибка при аутентификации. Пожалуйста, попробуйте позже."
                )

        @self.bot.message_handler(func=lambda message: True)
        def handle_unknown(message):
            self.bot.reply_to(
                message,
                "Для аутентификации используйте специальную ссылку с сайта."
            )


def run_telegram_bot():
    try:
        bot = DjangoTelegramBot()
        bot.auth_bot()
    except Exception as e:
        print(f"Критическая ошибка при запуске бота: {e}")


if __name__ == '__main__':
    run_telegram_bot()


from django import forms

class TelegramAuthenticationForm(forms.Form):
    username = forms.CharField(
        label='Username',
        widget=forms.TextInput(attrs={'class': 'form-control', 'placeholder': 'Введите username или Telegram ID'})
    )

    def clean(self):
        cleaned_data = super().clean()
        username = cleaned_data.get('username')
        if not username:
            raise forms.ValidationError('Требуется указать username или Telegram ID')
        return cleaned_data


from django.core.management.base import BaseCommand
from auth_tg.tg_bot import DjangoTelegramBot

class Command(BaseCommand):
    help = 'Запуск Telegram бота'

    def handle(self, *args, **kwargs):
        self.stdout.write(self.style.SUCCESS('Бот запущен...'))
        bot = DjangoTelegramBot()
        bot.auth_bot()
        self.stdout.write(self.style.SUCCESS('Бот выключен'))




from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.db import models
from django.utils import timezone
import uuid
from auth_tg.user_manager import AuthTgUserManager


class User(AbstractBaseUser, PermissionsMixin):
    username = models.CharField(max_length=150, null=True, blank=True, unique=True)
    tg_id = models.BigIntegerField(unique=True, verbose_name="Telegram ID")
    tg_username = models.CharField(max_length=255, blank=True, null=True, verbose_name="Telegram Username")
    tg_firstname = models.CharField(max_length=255, blank=True, null=True, verbose_name="Имя")
    tg_lastname = models.CharField(max_length=255, blank=True, null=True, verbose_name="Фамилия")
    tg_photo = models.URLField(max_length=500, blank=True, null=True, verbose_name="URL фото")
    created_at = models.DateTimeField(default=timezone.now, verbose_name="Время регистрации")

    is_staff = models.BooleanField(
        default=False,
        verbose_name="Staff status",
        help_text="Designates whether the user can log into this admin site."
    )
    is_active = models.BooleanField(
        default=True,
        verbose_name="Active",
        help_text="Designates whether this user should be treated as active."
    )
    is_superuser = models.BooleanField(
        default=False,
        verbose_name="Superuser status",
        help_text="Designates that this user has all permissions without explicitly assigning them."
    )

    objects = AuthTgUserManager()

    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = "Telegram пользователь"
        verbose_name_plural = "Telegram пользователи"
        swappable = 'AUTH_USER_MODEL'

    def __str__(self):
        return f"{self.tg_firstname} {self.tg_lastname} ({self.tg_username})"

    def save(self, *args, **kwargs):
        if not self.username:
            self.username = self.tg_username if self.tg_username else str(self.tg_id)
        super().save(*args, **kwargs)

    def get_photo_path(self):
        if not self.tg_photo:
            return None
        try:
            if '/photos/' in self.tg_photo:
                return 'photos/' + self.tg_photo.split('/photos/')[-1]
            return None
        except Exception:
            return None

    def has_perm(self, perm, obj=None):
        if self.is_superuser:
            return True
        if self.is_staff:
            return True
        return False

    def has_module_perms(self, app_label):
        return True if self.is_superuser else False

    def check_password(self, raw_password):
        return True

    def set_password(self, raw_password):
        pass


class TelegramAuthCode(models.Model):
    auth_code = models.UUIDField(default=uuid.uuid4, unique=True)
    created_at = models.DateTimeField(auto_now_add=True)
    is_used = models.BooleanField(default=False)
    user = models.ForeignKey(User, null=True, blank=True, on_delete=models.CASCADE)


from django.urls import path, re_path

from .views import *

urlpatterns = [
    path('tg/auth/profile/', TgAuthProfileView.as_view()),
    path('tg/auth/home', TgAuthHomeView.as_view()),
    re_path(r'^profile/photo/(?P<file_path>photos/[^/]+)$', TgAuthProfilePhotoView.as_view(),
            name='profile_photo'),
    path('logout/', TgAuthLogoutView.as_view(), name='logout'),
    path('api/telegram/login/', TgAuthTelegramLoginView.as_view(), name='telegram_login'),
    path('api/telegram/check-auth-status/', check_auth_status, name='check_auth_status'),
    path('auth/telegram/complete/', TgAuthTelegramCompleteView.as_view(), name='telegram_complete'),
]


from setuptools import setup, find_packages

setup(
    name='django-tgauth',
    version='0.1.3',
    packages=find_packages(include=['auth_tg', 'auth_tg.*']),
    include_package_data=True,
    package_data={
        'auth_tg': [
            'management/commands/*.py',
            'static/auth_tg/js/*.js',
            'templates/admin/*.html',
            'templates/auth_tg/*.html',
            'migrations/*.py'
        ],
    },
    license='MIT',
    description='App for Django and DRF which give an opportunity to login via TG.',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/hraon88/tgauth',
    author='hraon88',
    author_email='irm7700@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 5.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.12',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'Django>=5.1.5,<5.2',
        'djangorestframework>=3.15.2',
        'pyTelegramBotAPI>=4.26.0',
        'requests>=2.25.0',
    ],
    python_requires='>=3.12',
)